@@ -41,7 +41,7 @@ _LOG = messaging.get_logger(__name__)
 
 import dendropy
 
-PAUP_PATH = os.environ.get(metavar.DENDROPY_PAUP_PATH_ENVAR, "NONE")
+PAUP_PATH = os.environ.get(metavar.DENDROPY_PAUP_PATH_ENVAR, "paup")
 if PAUP_PATH == "NONE":
     DENDROPY_PAUP_INTEROPERABILITY = False
 else:
@@ -427,7 +427,7 @@ class PaupService(object):
         ``stage_list_taxa``, this parses out and returns a taxon block.
         """
         taxlabels = []
-        taxinfo_pattern = re.compile('\s*(\d+) (.*)\s+\-')
+        taxinfo_pattern = re.compile(r'\s*(\d+) (.*)\s+\-')
         idx = 0
         for line in paup_output:
             idx += 1
@@ -477,12 +477,12 @@ class PaupService(object):
         bipartition_freqs = {}
         bipartition_counts = {}
         tree_count = None
-        tree_count_pattern = re.compile('.*Majority-rule consensus of ([\d]*) tree.*', re.I)
+        tree_count_pattern = re.compile(r'.*Majority-rule consensus of ([\d]*) tree.*', re.I)
 
-        bipartition_section = re.compile('Bipartitions found in one or more trees and frequency of occurrence:')
-        bp_full_row_with_perc_col = re.compile('([\.|\*]+)\s+([\d\.]+)\s+([\d\.]*)%')
-        bp_full_row_with_no_perc_col = re.compile('([\.|\*]+)\s+([\d\.]+)')
-        bp_row = re.compile('([\.|\*]+).*')
+        bipartition_section = re.compile(r'Bipartitions found in one or more trees and frequency of occurrence:')
+        bp_full_row_with_perc_col = re.compile(r'([\.|\*]+)\s+([\d\.]+)\s+([\d\.]*)%')
+        bp_full_row_with_no_perc_col = re.compile(r'([\.|\*]+)\s+([\d\.]+)')
+        bp_row = re.compile(r'([\.|\*]+).*')
 
         # find tree count
         for idx, line in enumerate(paup_output):
@@ -644,7 +644,7 @@ def estimate_ultrametric_tree(
     if topology_tree is None:
         ultrametric_tree = estimate_tree(char_matrix,
                 tree_est_criterion="nj",
-                num_states=2,
+                num_subst=2,
                 unequal_base_freqs=False,
                 gamma_rates=False,
                 prop_invar=False,
@@ -674,8 +674,7 @@ def estimate_ultrametric_tree(
         paup_args["post_est_commands"] = post_est_commands
         paup_args["outtree_file"] = output_tree_filepath
         paup_block = paup_block % paup_args
-        paup_run = subprocess.Popen(['%s -n' % paup_path],
-                                    shell=True,
+        paup_run = subprocess.Popen([paup_path, "-n"],
                                     stdin=subprocess.PIPE,
                                     stdout=subprocess.PIPE)
         stdout, stderr = processio.communicate(paup_run, paup_block)
@@ -687,7 +686,7 @@ def estimate_ultrametric_tree(
 
 def estimate_tree(char_matrix,
                     tree_est_criterion="likelihood",
-                    num_states=6,
+                    num_subst=6,
                     unequal_base_freqs=True,
                     gamma_rates=True,
                     prop_invar=True,
@@ -701,7 +700,7 @@ def estimate_tree(char_matrix,
     Given a dataset, ``char_matrix``, estimates a tree using the given criterion.
     """
     paup_args = {
-        'nst': num_states,
+        'nst': num_subst,
         'basefreq' : unequal_base_freqs and 'estimate' or 'equal',
         'rates' : gamma_rates and 'gamma' or 'equal',
         'pinvar' : prop_invar and 'estimate' or '0',
@@ -734,7 +733,7 @@ def estimate_tree(char_matrix,
     """
     if tree_est_criterion.startswith("like"):
         paup_template += """\
-    lset tratio=estimate rmatrix=estimate nst=%(nst)s basefreq=%(basefreq)s rates=%(rates)s shape=estimate pinvar=%(pinvar)s userbrlens=yes;
+    lset tratio=estimate rmatrix=estimate nst=%(nst)s basefreq=%(basefreq)s rates=%(rates)s shape=estimate pinvar=%(pinvar)s ;
     """
     if tree_est_criterion not in ["nj", "upgma"] :
         paup_template += """\
@@ -751,7 +750,7 @@ def estimate_tree(char_matrix,
 
     paup_template += """\
     %(post_est_commands)s;
-    savetrees file=%(est_tree_file)s format=nexus root=yes brlens=yes taxablk=yes maxdecimals=20;
+    savetrees file=%(est_tree_file)s format=nexus brlens=yes taxablk=yes maxdecimals=20;
     """
     # paup_run = subprocess.Popen(['%s -n' % paup_path],
     #                             shell=True,
@@ -770,7 +769,7 @@ def estimate_tree(char_matrix,
 
 def estimate_model(char_matrix,
                     tree_model=None,
-                    num_states=6,
+                    num_subst=6,
                     unequal_base_freqs=True,
                     gamma_rates=True,
                     prop_invar=True,
@@ -785,7 +784,7 @@ def estimate_model(char_matrix,
     base_frequencies, alpha, prop_invar, etc. as well as likelihood.
     """
     paup_args = {
-        'nst': num_states,
+        'nst': num_subst,
         'basefreq' : unequal_base_freqs and 'estimate' or 'equal',
         'rates' : gamma_rates and 'gamma' or 'equal',
         'pinvar' : prop_invar and 'estimate' or '0',
@@ -830,20 +829,20 @@ def estimate_model(char_matrix,
     stdout, stderr = processio.communicate(paup_run, paup_template % paup_args)
     results = {}
     patterns = {
-        'likelihood' : re.compile('-ln L\s+([\d\.]+)'),
-        'rAC' : re.compile('  AC\s+([\d\.]+)'),
-        'rAG' : re.compile('  AG\s+([\d\.]+)'),
-        'rAT' : re.compile('  AT\s+([\d\.]+)'),
-        'rCG' : re.compile('  CG\s+([\d\.]+)'),
-        'rCT' : re.compile('  CT\s+([\d\.]+)'),
-        'rGT' : re.compile('  GT\s+([\d\.]+)'),
-        'kappa': re.compile('  kappa\s+([\d\.]+)'),
-        'prop_invar' : re.compile('P_inv\s+([\d\.]+)'),
-        'alpha' : re.compile('Shape\s+([\S]+)'),
-        'pA' : re.compile('  A\s+([\d\.]+)'),
-        'pC' : re.compile('  C\s+([\d\.]+)'),
-        'pG' : re.compile('  G\s+([\d\.]+)'),
-        'pT' : re.compile('  T\s+([\d\.]+)'),
+        'likelihood' : re.compile(r'-ln L\s+([\d\.]+)'),
+        'rAC' : re.compile(r'  AC\s+([\d\.]+)'),
+        'rAG' : re.compile(r'  AG\s+([\d\.]+)'),
+        'rAT' : re.compile(r'  AT\s+([\d\.]+)'),
+        'rCG' : re.compile(r'  CG\s+([\d\.]+)'),
+        'rCT' : re.compile(r'  CT\s+([\d\.]+)'),
+        'rGT' : re.compile(r'  GT\s+([\d\.]+)'),
+        'kappa': re.compile(r'  kappa\s+([\d\.]+)'),
+        'prop_invar' : re.compile(r'P_inv\s+([\d\.]+)'),
+        'alpha' : re.compile(r'Shape\s+([\S]+)'),
+        'pA' : re.compile(r'  A\s+([\d\.]+)'),
+        'pC' : re.compile(r'  C\s+([\d\.]+)'),
+        'pG' : re.compile(r'  G\s+([\d\.]+)'),
+        'pT' : re.compile(r'  T\s+([\d\.]+)'),
     }
     for value_name in patterns:
         results[value_name] = None
@@ -909,7 +908,7 @@ class PaupSession(processio.Session):
 
     EOC_FLAG = "@@@END-OF-COMMAND@@@"
     FLAG_DETECT = re.compile(r'^\s*%s\s*$' % EOC_FLAG, re.MULTILINE)
-    EOC_FLAG_STRIP = re.compile(r"^(paup>)*\s*(\[!)*" + EOC_FLAG + "(\])*\s*$", re.MULTILINE)
+    EOC_FLAG_STRIP = re.compile(r"^(paup>)*\s*(\[!)*" + EOC_FLAG + r"(\])*\s*$", re.MULTILINE)
     # FLAG_DETECT = re.compile(r'[^\[]\s*%s\s*[^\]]' % EOC_FLAG, re.MULTILINE)
 
     def __init__(self, paup_path=None):
