@@ -494,6 +494,509 @@ def birth_death_tree(birth_rate, death_rate, birth_rate_sd=0.0, death_rate_sd=0.
             nd.taxon = taxon
     return tree
 
+
+#NICOLA: new version of birth-death tree simulation with linear time cost
+#in number of tips instead of quadratic. To obtain this accelleration, however,
+#I have removed variation in birth and death rates.
+def fast_birth_death_tree(birth_rate, death_rate, **kwargs):
+    #NICOLA: we are not allowing variation in birth and death rate so to have
+    #increased efficiency.
+    birth_rate_sd=0.0
+    death_rate_sd=0.0
+
+    """
+    Returns a birth-death tree with birth rate specified by ``birth_rate``, and
+    death rate specified by ``death_rate``, with edge lengths in continuous (real)
+    units.
+
+    Tree growth is controlled by one or more of the following arguments, of which
+    at least one must be specified:
+
+        - If ``num_extant_tips`` is given as a keyword argument, tree is grown until the
+          number of EXTANT tips equals this number.
+        - If ``num_extinct_tips`` is given as a keyword argument, tree is grown until the
+          number of EXTINCT tips equals this number.
+        - If ``num_total_tips`` is given as a keyword argument, tree is grown until the
+          number of EXTANT plus EXTINCT tips equals this number.
+        - If 'max_time' is given as a keyword argument, tree is grown for
+          a maximum of ``max_time``.
+        - If ``gsa_ntax`` is given then the tree will be simulated up to this number of
+          EXTANT tips (or 0 tips), then a tree will be randomly selected from the
+          intervals which corresond to times at which the tree had exactly ``num_extant_tips``
+          leaves. This allows for simulations according to the "General
+          Sampling Approach" of Hartmann et al. (2010). If this option is
+          specified, then ``num_extant_tips`` MUST be specified and
+          ``num_extinct_tips`` and ``num_total_tips`` CANNOT be specified.
+
+    If more than one of the above is given, then tree growth will terminate when
+    *any* one of the termination conditions are met.
+
+    Parameters
+    ----------
+
+    birth_rate : float
+        The birth rate.
+    death_rate : float
+        The death rate.
+    birth_rate_sd : float
+        The standard deviation of the normally-distributed mutation added to
+        the birth rate as it is inherited by daughter nodes; if 0, birth rate
+        does not evolve on the tree.
+    death_rate_sd : float
+        The standard deviation of the normally-distributed mutation added to
+        the death rate as it is inherited by daughter nodes; if 0, death rate
+        does not evolve on the tree.
+
+    Keyword Arguments
+    -----------------
+
+    num_extant_tips: int
+        If specified, branching process is terminated when number of EXTANT
+        tips equals this number.
+    num_extinct_tips: int
+        If specified, branching process is terminated when number of EXTINCT
+        tips equals this number.
+    num_total_tips: int
+        If specified, branching process is terminated when number of EXTINCT
+        plus EXTANT tips equals this number.
+    max_time: float
+        If specified, branching process is terminated when time reaches or
+        exceeds this value.
+    gsa_ntax: int
+        The General Sampling Approach threshold for number of taxa. See above
+        for details.
+    tree : Tree instance
+        If given, then this tree will be used; otherwise a new one will be created.
+    taxon_namespace : TaxonNamespace instance
+        If given, then this will be assigned to the new tree, and, in addition,
+        taxa assigned to tips will be sourced from or otherwise created with
+        reference to this.
+    is_assign_extant_taxa : bool [default: True]
+        If False, then taxa will not be assigned to extant tips. If True
+        (default), then taxa will be assigned to extant tips. Taxa will be
+        assigned from the specified ``taxon_namespace`` or
+        ``tree.taxon_namespace``. If the number of taxa required exceeds the
+        number of taxa existing in the taxon namespace, new |Taxon| objects
+        will be created as needed and added to the taxon namespace.
+    is_assign_extinct_taxa : bool [default: True]
+        If False, then taxa will not be assigned to extant tips. If True
+        (default), then taxa will be assigned to extant tips. Taxa will be
+        assigned from the specified ``taxon_namespace`` or
+        ``tree.taxon_namespace``. If the number of taxa required exceeds the
+        number of taxa existing in the taxon namespace, new |Taxon| objects
+        will be created as needed and added to the taxon namespace. Note that
+        this option only makes sense if extinct tips are retained (specified via
+        'is_retain_extinct_tips' option), and will otherwise be ignored.
+    is_add_extinct_attr: bool [default: True]
+        If True (default), add an boolean attribute indicating whether or not a
+        node is an extinct tip or not. False will skip this. Name of attribute
+        is set by 'extinct_attr_name' argument, defaulting to 'is_extinct'.
+        Note that this option only makes sense if extinct tips are retained
+        (specified via 'is_retain_extinct_tips' option), and will otherwise be
+        ignored.
+    extinct_attr_name: str [default: 'is_extinct']
+        Name of attribute to add to nodes indicating whether or not tip is extinct.
+        Note that this option only makes sense if extinct tips are retained
+        (specified via 'is_retain_extinct_tips' option), and will otherwise be
+        ignored.
+    is_retain_extinct_tips : bool [default: False]
+        If True, extinct tips will be retained on tree. Defaults to False:
+        extinct lineages removed from tree.
+    repeat_until_success: bool [default: True]
+        Under some conditions, it is possible for all lineages on a tree to go
+        extinct. In this case, if this argument is given as |True| (the
+        default), then a new branching process is initiated. If |False|
+        (default), then a TreeSimTotalExtinctionException is raised.
+    rng: random.Random() or equivalent instance
+        A Random() object or equivalent can be passed using the ``rng`` keyword;
+        otherwise GLOBAL_RNG is used.
+
+    References
+    ----------
+
+    Hartmann, Wong, and Stadler "Sampling Trees from Evolutionary Models" Systematic Biology. 2010. 59(4). 465-476
+
+    """
+    if "assign_taxa" in kwargs:
+        deprecate.dendropy_deprecation_warning(
+                message="Deprecated: 'assign_taxa' will no longer be supported as an argument to this function. Use 'is_assign_extant_taxa' and/or 'is_assign_extinct_taxa' instead",
+                stacklevel=3)
+        a = kwargs.pop("assign_taxa")
+        kwargs["is_assign_extant_taxa"] = a
+        kwargs["is_assign_extant_taxa"] = a
+    if "ntax" in kwargs:
+        deprecate.dendropy_deprecation_warning(
+                message="Deprecated: 'ntax' is no longer supported as an argument to this function. Use one or more of the following instead: 'num_extant_tips', 'num_extinct_tips', 'num_total_tips', or 'max_time'",
+                stacklevel=3)
+        kwargs["num_extant_tips"] = kwargs.pop("ntax")
+    if (("num_extant_tips" not in kwargs)
+            and ("num_extinct_tips" not in kwargs)
+            and ("num_total_tips" not in kwargs)
+            and ("max_time" not in kwargs) ):
+        if "taxon_namespace" in kwargs:
+            ### cannot support legacy approach, b/c ``taxon_namespace`` may grow during function, leading to unpredictable behavior
+            # deprecate.dendropy_deprecation_warning(
+            #         preamble="Deprecated: The 'taxon_namespace' argument can no longer be used to specify a termination condition as a side-effect. Use one or more of the following instead with the length of the taxon namespace instance as a value: 'num_extant_tips', 'num_extinct_tips', or 'num_total_tips'",
+            #         old_construct="tree = birth_death_tree(\n    ...\n    taxon_namespace=taxon_namespace,\n    ...\n)",
+            #         new_construct="tree = birth_death_tree(\n    ...\n    taxon_namespace=taxon_namespace,\n    num_extant_tips=len(taxon_namespace),\n    ...\n)")
+            # kwargs["num_extant_tips"] = len(kwargs["taxon_namespace"])
+            raise ValueError("The 'taxon_namespace' argument can no longer be used to specify a termination condition as a side-effect."
+                             "Use one or more of the following instead with the length of the taxon namespace instance as a value: "
+                             "'num_extant_tips', 'num_extinct_tips', or 'num_total_tips'.\n"
+                             "That is, instead of:\n\n"
+                             "    tree = birth_death_tree(\n        ...\n        taxon_namespace=taxon_namespace,\n        ...\n    )\n\n"
+                             "Use:\n\n"
+                             "    ntax = len(taxon_namespace)\n    tree = birth_death_tree(\n        ...\n        taxon_namespace=taxon_namespace,\n        num_extant_tips=ntax,\n        ...\n    )\n"
+                             "\nOr (recommended):\n\n"
+                             "    tree = birth_death_tree(\n        ...\n        taxon_namespace=taxon_namespace,\n        num_extant_tips=100,\n        ...\n    )\n"
+                             "\nNote that the taxon namespace instance size may grow during any particular call of the function depending on taxon assignment/creation settings, so"
+                             " for stable and predictable behavor it is important to take a snapshot of the desired taxon namespace size before any call of the function, or, better yet"
+                             " simply pass in a constant value."
+                             )
+        else:
+            raise ValueError("One or more of the following must be specified: 'num_extant_tips', 'num_extinct_tips', or 'max_time'")
+    target_num_extant_tips = kwargs.pop("num_extant_tips", None)
+    target_num_extinct_tips = kwargs.pop("num_extinct_tips", None)
+    target_num_total_tips = kwargs.pop("num_total_tips", None)
+    max_time = kwargs.pop('max_time', None)
+    gsa_ntax = kwargs.pop('gsa_ntax', None)
+    is_add_extinct_attr = kwargs.pop('is_add_extinct_attr', True)
+    extinct_attr_name = kwargs.pop('extinct_attr_name', 'is_extinct')
+    is_retain_extinct_tips = kwargs.pop('is_retain_extinct_tips', False)
+    is_assign_extant_taxa = kwargs.pop('is_assign_extant_taxa', True)
+    is_assign_extinct_taxa = kwargs.pop('is_assign_extinct_taxa', True)
+    repeat_until_success = kwargs.pop('repeat_until_success', True)
+
+    tree = kwargs.pop("tree", None)
+    taxon_namespace = kwargs.pop("taxon_namespace", None)
+
+    rng = kwargs.pop('rng', GLOBAL_RNG)
+
+    ignore_unrecognized_keyword_arguments = kwargs.pop('ignore_unrecognized_keyword_arguments', False)
+    if kwargs and not ignore_unrecognized_keyword_arguments:
+        raise ValueError("Unsupported keyword arguments: {}".format(kwargs.keys()))
+
+    terminate_at_full_tree = False
+
+    if gsa_ntax is None:
+        terminate_at_full_tree = True
+        # gsa_ntax = 1 + target_num_taxa
+    elif target_num_extant_tips is None:
+        raise ValueError("If 'gsa_ntax' is specified, 'num_extant_tips' must be specified")
+    elif target_num_extinct_tips is not None:
+        raise ValueError("If 'gsa_ntax' is specified, 'num_extinct_tups' cannot be specified")
+    elif target_num_total_tips is not None:
+        raise ValueError("If 'gsa_ntax' is specified, 'num_total_tips' cannot be specified")
+    elif gsa_ntax < target_num_extant_tips:
+        raise ValueError("'gsa_ntax' ({}) must be greater than 'num_extant_tips' ({})".format(gsa_ntax, target_num_extant_tips))
+
+    # initialize tree
+    if tree is not None:
+        if taxon_namespace is not None:
+            assert tree.taxon_namespace is taxon_namespace
+        else:
+            taxon_namespace = tree.taxon_namespace
+        extant_tips = []
+        extinct_tips = []
+        for nd in tree:
+            if not nd._child_nodes:
+                if getattr(nd, extinct_attr_name, False):
+                    extant_tips.append(nd)
+                    #NICOLA: the reason for this will become clear later
+                    nd.edge.length=-nd.edge.length
+                    if is_add_extinct_attr:
+                        setattr(nd, extinct_attr_name, False)
+                else:
+                    extinct_tips.append(nd)
+                    if is_add_extinct_attr:
+                        setattr(nd, extinct_attr_name, True)
+            elif is_add_extinct_attr:
+                setattr(nd, extinct_attr_name, None)
+    else:
+        if taxon_namespace is None:
+            taxon_namespace = dendropy.TaxonNamespace()
+        tree = dendropy.Tree(taxon_namespace=taxon_namespace)
+        tree.is_rooted = True
+        tree.seed_node.edge.length = 0.0
+        tree.seed_node.birth_rate = birth_rate
+        tree.seed_node.death_rate = death_rate
+        if is_add_extinct_attr:
+            setattr(tree.seed_node, extinct_attr_name, False)
+        extant_tips = [tree.seed_node]
+        extinct_tips = []
+    initial_extant_tip_set = list(extant_tips)
+    initial_extinct_tip_set = list(extinct_tips)
+    #NICOLA: in case we need a restart, let's record the initial branch lengths.
+    initial_lengths=[]
+    for nd in initial_extant_tip_set:
+        initial_lengths.append(nd.edge.length)
+
+    total_time = 0
+
+    # for the GSA simulations targetted_time_slices is a list of tuple
+    #   the first element in the tuple is the duration of the amount
+    #   that the simulation spent at the (targetted) number of taxa
+    #   and a list of edge information. The list of edge information includes
+    #   a list of terminal edges in the tree and the length for that edge
+    #   that marks the beginning of the time slice that corresponds to the
+    #   targetted number of taxa.
+    targetted_time_slices = []
+
+    while True:
+        if gsa_ntax is None:
+            if target_num_extant_tips is not None and len(extant_tips) >= target_num_extant_tips:
+                #NICOLA: here and in the other cases, update the branch length so to
+                #represent the true branch length (explained better below).
+                for nd in extant_tips:
+                        nd.edge.length=total_time-nd.edge.length
+                break
+            if target_num_extinct_tips is not None and len(extinct_tips) >= target_num_extinct_tips:
+                for nd in extant_tips:
+                        nd.edge.length=total_time-nd.edge.length
+                break
+            if target_num_total_tips is not None and (len(extant_tips) + len(extinct_tips)) >= target_num_total_tips:
+                for nd in extant_tips:
+                        nd.edge.length=total_time-nd.edge.length
+                break
+            if max_time is not None and total_time >= max_time:
+                for nd in extant_tips:
+                        nd.edge.length=total_time-nd.edge.length
+                break
+        elif len(extant_tips) >= gsa_ntax:
+                for nd in extant_tips:
+                        nd.edge.length=total_time-nd.edge.length
+                break
+
+
+        # get vector of birth/death probabilities, and
+        # associate with nodes/events
+        #NICOLA: replaced this
+   #      event_rates = []
+#         event_nodes = []
+#         for nd in extant_tips:
+#             if not hasattr(nd, 'birth_rate'):
+#                 nd.birth_rate = birth_rate
+#             if not hasattr(nd, 'death_rate'):
+#                 nd.death_rate = death_rate
+#             event_rates.append(nd.birth_rate)
+#             event_nodes.append((nd, True)) # birth event = True
+#             event_rates.append(nd.death_rate)
+#             event_nodes.append((nd, False)) # birth event = False; i.e. death
+
+
+        # get total probability of any birth/death
+        #rate_of_any_event = sum(event_rates)
+        #NICOLA: instead, assume that each node has the same birth and death rate, and sample
+        #from the total rate and then sample one node at random from extant ones.
+        #this has constant cost instead of linear in the number of extant taxa.
+        rate_of_any_event=len(extant_tips)*(birth_rate+death_rate)
+
+        # waiting time based on above probability
+        waiting_time = rng.expovariate(rate_of_any_event)
+
+        if ( (gsa_ntax is not None)
+                and (len(extant_tips) == target_num_extant_tips)
+                ):
+            edge_and_start_length = []
+            for nd in extant_tips:
+                e = nd.edge
+                #NICOLA: modified accordingly
+                edge_and_start_length.append((e, total_time-e.length))
+            targetted_time_slices.append((waiting_time, edge_and_start_length))
+            if terminate_at_full_tree:
+                #NICOLA: terminal update as before
+                for nd in extant_tips:
+                        nd.edge.length=total_time-nd.edge.length
+                break
+
+        # add waiting time to nodes
+        #Nicola: this is the part that makes it quadratic in ntax.
+        #instead, we use a trick of initializing nd.edge.length to the time of node creation,
+        # and then we update it to the correct branch length when the node is closed or simulations are terminated.
+        #for nd in extant_tips:
+        #    try:
+        #        nd.edge.length += waiting_time
+        #    except TypeError:
+        #        nd.edge.length = waiting_time
+        total_time += waiting_time
+
+        # if event occurs within time constraints
+        if max_time is None or total_time <= max_time:
+            # normalize probability
+            #for i in range(len(event_rates)):
+            #    event_rates[i] = event_rates[i]/rate_of_any_event
+            # select node/event and process
+            #nd, birth_event = probability.weighted_choice(event_nodes, event_rates, rng=rng)
+            #NICOLA: instead of sampling from a categorical distribution (requiring linear time in ntax)
+            #sample a random integer to represent the chosen taxon, and sample if birth or death event.
+            taxI=rng.randint(0, len(extant_tips)-1)
+            if rng.random()< birth_rate/(birth_rate+death_rate):
+                birth_event=True
+            else:
+                birth_event=False
+            nd=extant_tips[taxI]
+            #extant_tips.remove(nd)
+            if birth_event:
+                if is_add_extinct_attr:
+                    setattr(nd, extinct_attr_name, None)
+                c1 = nd.new_child()
+                c2 = nd.new_child()
+                extant_tips[taxI]=c1
+                #NICOLA: we initialize branch lengths to total time;
+                #this may not make sense at first, but when we are done with a node, we
+                #update the branch length to the new current time minus the time of node creation
+                #which gives us the branch length. The advantage is that this takes linear time
+                #instead of quadratic.
+                c1.edge.length = total_time
+                c2.edge.length = total_time
+                nd.edge.length = total_time-nd.edge.length
+                #c1.edge.length = 0
+                #c2.edge.length = 0
+                #NICOLA: here, for speed, we don't allow variation in birth rates.
+                #It could be possible to allow it, but it would take ntax*log(ntax) time,
+                #which wouldn't be bad, but it would also require a more complicated approach.
+                #c1.birth_rate = nd.birth_rate + rng.gauss(0, birth_rate_sd)
+                #c1.death_rate = nd.death_rate + rng.gauss(0, death_rate_sd)
+                #c2.birth_rate = nd.birth_rate + rng.gauss(0, birth_rate_sd)
+                #c2.death_rate = nd.death_rate + rng.gauss(0, death_rate_sd)
+                c1.birth_rate = nd.birth_rate
+                c1.death_rate = nd.death_rate
+                c2.birth_rate = nd.birth_rate
+                c2.death_rate = nd.death_rate
+                #extant_tips.append(c1)
+                extant_tips.append(c2)
+            else:
+                del extant_tips[taxI]
+                if len(extant_tips) > 0:
+                    extinct_tips.append(nd)
+                    if is_add_extinct_attr:
+                        setattr(nd, extinct_attr_name, None)
+                else:
+                    # total extinction
+                    if (gsa_ntax is not None):
+                        if (len(targetted_time_slices) > 0):
+                            break
+                    if not repeat_until_success:
+                        raise TreeSimTotalExtinctionException()
+                    # We are going to basically restart the simulation because
+                    # the tree has gone extinct (without reaching the specified
+                    # ntax)
+                    extant_tips = list(initial_extant_tip_set)
+                    extinct_tips = list(initial_extinct_tip_set)
+                    ndIndex=0
+                    for nd in extant_tips:
+                        if is_add_extinct_attr:
+                            setattr(nd, extinct_attr_name, False)
+                        nd.clear_child_nodes()
+                        #NICOLA: restore branch length to original value
+                        nd.edge.length=initial_lengths[ndIndex]
+                        ndIndex+=1
+                    total_time = 0
+
+
+    if gsa_ntax is not None:
+        total_duration_at_target_n_tax = 0.0
+        for i in targetted_time_slices:
+            total_duration_at_target_n_tax += i[0]
+        r = rng.random()*total_duration_at_target_n_tax
+        selected_slice = None
+        for n, i in enumerate(targetted_time_slices):
+            r -= i[0]
+            if r < 0.0:
+                selected_slice = i
+        assert(selected_slice is not None)
+        edges_at_slice = selected_slice[1]
+        last_waiting_time = selected_slice[0]
+
+        for e, prev_length in edges_at_slice:
+            daughter_nd = e.head_node
+            for nd in daughter_nd.child_nodes():
+                nd._parent_node = None
+                try:
+                    extinct_tips.remove(nd)
+                except ValueError:
+                    pass
+                try:
+                    extant_tips.remove(nd)
+                except ValueError:
+                    pass
+                for desc in nd.preorder_iter():
+                    try:
+                        extant_tips.remove(desc)
+                    except ValueError:
+                        pass
+            daughter_nd.clear_child_nodes()
+            try:
+                extinct_tips.remove(daughter_nd)
+            except ValueError:
+                pass
+            extant_tips.append(daughter_nd)
+            if is_add_extinct_attr:
+                setattr(daughter_nd, extinct_attr_name, False)
+            e.length = prev_length + last_waiting_time
+
+    if not is_retain_extinct_tips:
+        processed_nodes = set()
+        for nd in list(extinct_tips):
+            if nd in processed_nodes:
+                continue
+            processed_nodes.add(nd)
+            try:
+                extinct_tips.remove(nd)
+            except ValueError:
+                pass
+            assert not nd._child_nodes
+            while (nd.parent_node is not None) and (len(nd.parent_node._child_nodes) == 1):
+                nd = nd.parent_node
+                processed_nodes.add(nd)
+            tree.prune_subtree(nd, suppress_unifurcations=False)
+    tree.suppress_unifurcations()
+
+    if is_assign_extant_taxa or is_assign_extinct_taxa:
+        taxon_pool = [t for t in taxon_namespace]
+        rng.shuffle(taxon_pool)
+        taxon_pool_labels = set([t.label for t in taxon_pool])
+
+        ### ONLY works if in GSA sub-section we remove ALL extant and
+        ### extinct nodes beyond time slice: expensive
+        ### Furthermore, main reason to use this approach is to have different
+        ### label prefixes for extinct vs. extant lineages, but the second time
+        ### this function is called with the same taxon namespace or any time
+        ### this function is called with a populated taxon namespace, that
+        ### aesthetic is lost.
+        # node_pool_labels = ("T", "X")
+        # for node_pool_idx, node_pool in enumerate((extant_tips, extinct_tips)):
+        #     for node_idx, nd in enumerate(node_pool):
+        #         if taxon_pool:
+        #             taxon = taxon_pool.pop()
+        #         else:
+        #             taxon = taxon_namespace.require_taxon("{}{}".format(node_pool_labels[node_pool_idx], node_idx+1))
+        #         nd.taxon = taxon
+        #         assert not nd._child_nodes
+
+        tlabel_counter = 0
+        leaf_nodes = tree.leaf_nodes()
+        rng.shuffle(leaf_nodes)
+        for nd_idx, nd in enumerate(leaf_nodes):
+            if not is_assign_extant_taxa and nd in extant_tips:
+                continue
+            if not is_assign_extant_taxa and nd in extinct_tips:
+                continue
+            if taxon_pool:
+                taxon = taxon_pool.pop()
+            else:
+                while True:
+                    tlabel_counter += 1
+                    label = "{}{}".format("T", tlabel_counter)
+                    if label not in taxon_pool_labels:
+                        break
+                taxon = taxon_namespace.require_taxon(label=label)
+                taxon_pool_labels.add(label)
+            nd.taxon = taxon
+    return tree
+
+
+
 def discrete_birth_death_tree(birth_rate, death_rate, birth_rate_sd=0.0, death_rate_sd=0.0, **kwargs):
     """
     Returns a birth-death tree with birth rate specified by ``birth_rate``, and
